/*******************************************************************************
 * Copyright (c) 2005 Intel Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Guru Nagarajan,Intel - Initial API and implementation
 *
 * $Id: TPTPAgent.java,v 1.18 2005/11/07 19:23:35 gnagarajan Exp $ 
 *******************************************************************************/ 
package org.eclipse.tptp.platform.execution.samples;

import org.eclipse.tptp.platform.execution.client.agent.*;
import org.eclipse.tptp.platform.execution.client.core.*;
import org.eclipse.tptp.platform.execution.util.*;
import org.w3c.dom.*;
/**
 * This sample connects to an Agent Controller running on the local host
 * and performs a series of agent related operations
 * <Pre>
 * 		i) Query Available Agents
 * 	   ii) Query Running Agents
 * 	   iii)Obtain Agent Reference with Controller Access
 * 	   iv) Release Agent Control
 * </Pre>
 * @author gnagaraj
 *
 */

public class TPTPAgent
{
	public static void main(String[] args)
	{
	    //Connect to the host where the Agent Controller is running
	    //See the Getting started guide for TPTP 4.0 
		String localhost = "localhost";
		//This port number corresponds to the port where the Agent Controller
		//listens/accepts connections. This is defined in the Agent Controller
		//serviceconfig.xml file in the <AgentController_INSTALL_HOME>/config
		int port = 10006;
		// Get the Command Line Parameters
		if (args.length > 0)
		{
			if (args.length >= 1)
			{
				localhost = args[0];
			}
			if (args.length >=2)
			{
				port = (new Integer(args[1])).intValue();
			}
		}
		
		//Create a Node that represents the target agent controller
		INode SampleNode = null;
		//Create the client Agent Controller representation object
		IAgentController ac = null;
		//Pass the connection info the client
		ConnectionInfo connInfo = new ConnectionInfo();
		
		try
		{
			//Create a Node that represents the target machine and agent controller
			SampleNode = NodeFactory.createNode(localhost);
			//Set the connection parameters required 
			connInfo.setHostName(localhost);
			connInfo.setPort(port);
			//connInfo.setAgentControllerNew(true);

			if (SampleNode != null)
			{
				// Connect to the Agent Controller, a successful connection returns 
			    //a client side Agent Controller object that provides various 
			    //methods to perform TPTP tasks.
				System.out.println("Connecting to AC..");
				ac = SampleNode.connect(connInfo);

				System.out.println("Connected to " + localhost + " at port number " + port);
				boolean _done = false;
			 do{
				System.out.println("---------------------------------------");
				System.out.println("Querying Available Agents ...");
				String[] availAgents = ac.queryAvailableAgents();
				if(availAgents != null) {
				for(int i = 0; i < availAgents.length; i++)
				{
					System.out.println("Available Agents ::" + availAgents[i]);
				}
				}
				
				System.out.println("---------------------------------------");
				System.out.println("Querying Available Agents by type...");
				String[] agentsType = {"org.eclipse.tptp.TimeCollector", "org.eclipse.tptp.FileTransfer"};
				String[] availAgentsByType = ac.queryAvailableAgents(agentsType);
				if(availAgentsByType != null) {
				for(int i = 0; i < availAgentsByType.length; i++)
				{
					System.out.println("Available Agents ::" + availAgentsByType[i]);
				}
				}
				
				System.out.println("---------------------------------------");
				System.out.println("Querying Running Agents...");
				IAgent[] RetAgentArr = ac.queryRunningAgents();
				if (RetAgentArr != null)
				{
					System.out.println("# of Running Agents ::" + RetAgentArr.length);
					for(int i = 0; i < RetAgentArr.length; i++)
					{
						System.out.println("Running Agents ::" + RetAgentArr[i].getName());
					}
				}
				System.out.println("---------------------------------------");
				System.out.println("Querying Running Agents by type ...");
				String[] RunningAgentsType = {"org.eclipse.tptp.TimeCollector"};
				IAgent[] RetAgentsByTypeArr = ac.queryRunningAgents(RunningAgentsType);
				
				if (RetAgentsByTypeArr != null)
				{
					System.out.println("# of Running Agents ::" + RetAgentsByTypeArr.length);
					for(int i = 0; i < RetAgentsByTypeArr.length; i++)
					{
						System.out.println("Running Agents by type::" + RetAgentsByTypeArr[i].getName());
					}
				}
				System.out.println("---------------------------------------");
				IAgent agent = ac.getAgent("org.eclipse.tptp.TimeCollector",TPTPAgentAccess.TPTP_CONTROLLER_ACCESS);
				if(agent != null){ String agentMetadata = agent.getAgentMetaData(); 
				System.out.println("Agent MetaData ::" + agentMetadata);
				}
				if(agent == null){ System.out.println("Agent unavailable ");return;}
				System.out.println("Agent ID for the agent ac " + agent.getAgentMode());
				if( agent.requestControl())
				System.out.println("Agent ID for the agent ac true ");
				agent.releaseControl();
				System.out.println("Agent ID for the agent ac true " +agent.getAgentMode());
				agent.releaseAccess();
				
				_done = true;
				
			 }while(!_done);
				
			}
			else
			{
				System.out.println("Error occurred while creating the Node for " + localhost);
			}
			
		}
		catch(Exception exp)
		{
			System.out.println("Error occurred while connecting to " + localhost + ":" + exp);
		}		
	}
}
